/* OscThread -- thread for making an oscillator go go go
 *	this one has a filter!
 *
 * 		Brad Garton, spring 2003
 *
*/
import com.softsynth.jsyn.*;

class OscThread extends Thread
{
	SawtoothOscillator	theOsc;
	SawtoothOscillator	theOsc2;
	DelayUnit		theDelay;
	AddUnit			theoscadder;
	SynthEnvelope		theEnv;
	EnvelopePlayer		theEnvPlayer;
	double [] envdata =
	{
		0.0, 0.0,
		0.5, 1.0,
		1.0, 1.0,
		0.5, 0.0
	};
	MultiplyUnit		theMult;
	LineOut			oscOut;
	Filter_LowPass		theFilt;

	public OscThread() throws SynthException
	{
		theOsc = new SawtoothOscillator();
		theOsc2 = new SawtoothOscillator();
		theoscadder = new AddUnit();
		theDelay = new DelayUnit(0.1);

		theFilt = new Filter_LowPass();

		// envdata has already been set up
		theEnv = new SynthEnvelope(envdata);
		theEnvPlayer = new EnvelopePlayer();

		theMult = new MultiplyUnit();
		oscOut = new LineOut();

		// connect 'em up!
		theOsc.output.connect(theoscadder.inputA);
		theOsc2.output.connect(theoscadder.inputB);
		theoscadder.output.connect(theFilt.input);
		theFilt.output.connect(theMult.inputA);
		theEnvPlayer.output.connect(theMult.inputB);

		// now in *stereo*, thanks to the delay!
		theMult.output.connect(0, oscOut.input, 0);
		theMult.output.connect(theDelay.input);
		theDelay.output.connect(0, oscOut.input, 1);
	}

	public void run()
	{
		try
		{
			theOsc.start();
			theOsc2.start();
			theoscadder.start();
			theDelay.start();
			theFilt.start();
			theEnvPlayer.start();
			theMult.start();
			oscOut.start();
		} catch(SynthException e) {
			SynthAlert.showError(e);
		}
	}

	public void filtset(int start, double amp, double freq, double Q)
	{
		// we could have passed these in as part of the go() method
		theFilt.amplitude.set(start, amp);
		theFilt.frequency.set(start, freq);
		theFilt.Q.set(start, Q);
	}


	public void go(int start, double dur, double freq, double amp, double rise, double decay)
	{
		double [] envdata2 =
		{
			0.0, 0.0,
			0.5, 1.0,
			1.0, 1.0,
			0.5, 0.0
		};

		// start in ticks, dur/rise/decay in seconds, freq in Hz
		try
		{
			theOsc.frequency.set(start, freq);
			theOsc.amplitude.set(start, amp);
			theOsc2.frequency.set(start, freq+1.0);
			theOsc2.amplitude.set(start, amp);

			envdata2[2] = rise;
			envdata2[4] = dur-(rise+decay);
			envdata2[6] = decay;
			theEnv.write(envdata2);

			theEnvPlayer.envelopePort.clear(start);
			theEnvPlayer.envelopePort.queue(start, theEnv);
		} catch(SynthException e) {
			SynthAlert.showError(e);
		}
	}
}
